@@ -193,7 +193,6 @@ uActionCamera eCamera::se_glanceRight("GLANCE_RIGHT",-100);
 
 uActionCamera eCamera::se_glanceLeft("GLANCE_LEFT",-110);
 
-
 uActionCamera eCamera::se_lookDown("BANK_DOWN",
                                    -120,
                                    uAction::uINPUT_ANALOG);
@@ -213,6 +212,11 @@ uActionCamera eCamera::se_lookLeft("LOOK_LEFT",
 
 uActionCamera eCamera::se_switchView("SWITCH_VIEW", -160);
 
+uActionTooltip eCamera::se_glanceBackTooltip( eCamera::se_glanceBack, 1 );
+uActionTooltip eCamera::se_glanceRightTooltip( eCamera::se_glanceRight, 3 );
+uActionTooltip eCamera::se_glanceLeftTooltip( eCamera::se_glanceLeft, 3 );
+uActionTooltip eCamera::se_switchViewTooltip( eCamera::se_switchView, 2 );
+
 
 static REAL s_startFollowX = -30, s_startFollowY = -30, s_startFollowZ = 80;
 static REAL s_startSmartX = 10, s_startSmartY = 30, s_startSmartZ = 2;
@@ -608,9 +612,9 @@ bool eCamera::Act(uActionCamera *Act,REAL x){
     pos=pos+dir*mf*.25+dir.Turn(eCoord(0,ml*.25));
 
     fov/=zi;
-    if (fov>120) fov=120;
+    if (fov>160 && zi < 1) fov=160;
 
-    if (fov<30) fov=30;
+    if (fov<30 && zi > 1) fov=30;
 
 
     switch(mode){
@@ -2098,8 +2102,7 @@ void eCamera::s_Timestep(eGrid *grid, REAL time){
 #ifndef DEDICATED
 
 void eCamera::SoundMix(Uint8 *dest,unsigned int len){
-    if (!this)
-        return;
+    tASSERT_THIS();
 
     if (id>=0){
         eGameObject *c=Center();
@@ -2114,6 +2117,9 @@ void eCamera::SoundMix(Uint8 *dest,unsigned int len){
 
 
 void eCamera::SoundMixGameObject(Uint8 *dest,unsigned int len,eGameObject *go){
+    if(!go)
+        return;
+    
     eCoord vec((go->pos-pos).Turn(dir.Conj()));
     REAL dist_squared=vec.NormSquared()+(z-go->z)*(z-go->z);
 
