@@ -43,8 +43,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <map>
 
 uActionPlayer eGameObject::se_turnRight("CYCLE_TURN_RIGHT", -10);
+static uActionTooltip se_turnRightTooltip( eGameObject::se_turnRight, 11, &ePlayer::VetoActiveTooltip );
 
 uActionPlayer eGameObject::se_turnLeft("CYCLE_TURN_LEFT", -10);
+static uActionTooltip se_turnLeftTooltip( eGameObject::se_turnLeft, 10, &ePlayer::VetoActiveTooltip );
 
 
 // entry and deletion in the list of all gameObjects
@@ -119,6 +121,7 @@ void eGameObject::DoRemoveFromGame()
 eGameObject::eGameObject(eGrid *g,const eCoord &p,const eCoord &d,eFace *currentface,bool autodel)
         :autodelete(autodel),pos(p),dir(d),z(0),grid(g){
     tASSERT(g);
+    urgentSimulationRequested_=false;
     currentFace=currentface;
     lastTime=se_GameTime();
     id=-1;
@@ -698,10 +701,6 @@ bool eGameObject::TimestepThis(REAL currentTime,eGameObject *c){
 
     REAL maxstep=.2;
 
-    // don't do a thing if the timestep is too small
-    if (fabs(currentTime - c->lastTime) < .001)
-        return false;
-
     // be more careful when going back
     if (currentTime<c->lastTime)
         maxstep=.1;
@@ -728,6 +727,8 @@ bool eGameObject::TimestepThis(REAL currentTime,eGameObject *c){
                 c->InteractWith(c->grid->gameObjectsInteresting(j),currentTime,0);
 
         REAL timeThisStep = lastTime+i*(currentTime-lastTime)/number_of_steps;
+
+        c->urgentSimulationRequested_ = false;
         ret = ret || c->Timestep(timeThisStep);
         c->FindCurrentFace();
 
@@ -735,6 +736,14 @@ bool eGameObject::TimestepThis(REAL currentTime,eGameObject *c){
         if ( 2 * c->lastTime < timeThisStep + lastTime )
             break;
     }
+    for(int timeout = 10; timeout >= 0 && c->urgentSimulationRequested_; --timeout )
+    {
+        // simulate on while events are pending
+        c->urgentSimulationRequested_ = false;
+        ret = ret || c->Timestep(currentTime);
+        c->FindCurrentFace();
+    }
+
 #ifdef DEBUG
     c->grid->Check();
 #endif
@@ -743,7 +752,7 @@ bool eGameObject::TimestepThis(REAL currentTime,eGameObject *c){
 }
 
 #ifdef DEDICATED
-static REAL se_maxSimulateAhead = .01f;
+static REAL se_maxSimulateAhead = .1f;
 static tSettingItem<REAL> se_maxSimulateAheadConf( "MAX_SIMULATE_AHEAD", se_maxSimulateAhead );
 #endif
 
@@ -778,23 +787,14 @@ void eGameObject::TimestepThisWrapper(eGrid * grid, REAL currentTime, eGameObjec
 #endif
         simTime -= c->Lag();
 
+    REAL maxSimTime = simTime;
 #ifdef DEDICATED
-    REAL nextTime = c->NextInterestingTime();
-
-    // store the time left to simulate
-    se_maxSimulateAheadLeft = simTime + se_maxSimulateAhead - nextTime;
-    if ( se_maxSimulateAheadLeft < 0 )
-        se_maxSimulateAheadLeft = 0;
+    se_maxSimulateAheadLeft = se_maxSimulateAhead+se_lazyLag;
 
     REAL lagThreshold = c->LagThreshold();
-    if ( simTime - lagThreshold < nextTime && nextTime < simTime + se_maxSimulateAhead )
+    if( !c->urgentSimulationRequested_ )
     {
-        // something interesting is going to happen, see what it is
-        simTime = nextTime;
-    }
-    else
-    {
-        // add an extra portion of lag compensation
+        // nothing interesting happening. add an extra portion of lag compensation
         simTime -= lagThreshold;
 
         if ( simTime < c->LastTime() + minTimestep )
@@ -803,6 +803,11 @@ void eGameObject::TimestepThisWrapper(eGrid * grid, REAL currentTime, eGameObjec
             return;
         }
     }
+    else
+    {
+        maxSimTime += se_maxSimulateAheadLeft;
+    }
+
 #endif
 
     // check for teleports out of arena bounds
@@ -815,7 +820,7 @@ void eGameObject::TimestepThisWrapper(eGrid * grid, REAL currentTime, eGameObjec
     }
 
     // only simulate forward here
-    if ( simTime > c->lastTime )
+    if ( maxSimTime > c->lastTime )
     {
         if (TimestepThis(simTime,c))
         {
@@ -880,6 +885,9 @@ void eGameObject::RenderAll(eGrid *grid, const eCamera *cam){
                 // but the small flickering error is to be tolerated, especially
                 // since alpha blended game objects tend to gently fade in.
                 int firstAlphaID = firstAlpha->id;
+
+                eSoundLocker locker;
+	       
                 grid->gameObjects.Remove(firstAlpha,firstAlpha->id);
                 grid->gameObjects.Add(firstAlpha,firstAlpha->id);
                 grid->gameObjects.Remove(object,object->id);
