@@ -490,6 +490,12 @@ unsigned short next_free(){
                 sn_SendPlanned();
                 //	st_Breakpoint();
                 tAdvanceFrame(1000000);
+
+                // check for user abort, but just make it time out faster
+                if ( tConsole::Idle(false) )
+                {
+                    timeout -= 10;
+                }
             }
             if (tSysTimeFloat()>=timeout)
                 tERR_ERROR_INT("Not enough nNetObject IDs to distribute. Sorry!\n");
@@ -555,14 +561,11 @@ nDescriptor& nNetObject::CreatorDescriptor() const{
 */
 
 void nNetObject::AddRef(){
-    tASSERT ( this );
+    tASSERT_THIS();
 
-    if ( this )
-    {
-        tASSERT( refCtr_ >= 0 );
-        refCtr_++;
-        tASSERT( refCtr_ >= 0 );
-    }
+    tASSERT( refCtr_ >= 0 );
+    refCtr_++;
+    tASSERT( refCtr_ >= 0 );
 }
 
 void nNetObject::ReleaseOwnership(){
@@ -582,9 +585,9 @@ void nNetObject::TakeOwnership(){
 }
 
 void nNetObject::Release(){
-    tASSERT( this );
+    tASSERT_THIS();
 
-    if (this){
+    {
         if (refCtr_>0)
             refCtr_--;
         else
@@ -892,7 +895,7 @@ static void net_destroy_handler(nMessage &m){
         info.timeout=tSysTimeFloat()+nDeletedTimeout;
 
         // notify object of pending deletion
-        if (nNetObject *no=sn_netObjects[id])
+        if (tJUST_CONTROLLED_PTR<nNetObject> no=sn_netObjects[id])
         {
             tASSERT( !no->Owned() );
 
@@ -1155,7 +1158,15 @@ nNetObject *nNetObject::Object(int i){
 
     bool printMessage=true;
     while (sn_Connections[0].socket &&
-            NULL==(ret=sn_netObjects[i]) && timeout >tSysTimeFloat()){ // wait until it is spawned
+            NULL==(ret=sn_netObjects[i]) && timeout >tSysTimeFloat())
+    { // wait until it is spawned
+
+        // check for user abort, but just make it time out faster
+        if ( tConsole::Idle(false) )
+        {
+            timeout -= 10;
+        }
+
         if (tSysTimeFloat()>timeout-(totalTimeout + printMessageTimeout))
         {
             if (printMessage)
@@ -1481,6 +1492,9 @@ static void net_sync_handler(nMessage &m){
 static nDescriptor net_sync(24,net_sync_handler,"net_sync");
 
 bool nNetObject::AcceptClientSync() const{
+    return AcceptClientSyncStatic();
+}
+bool nNetObject::AcceptClientSyncStatic(){
     return false;
 }
 
@@ -1530,7 +1544,7 @@ void nNetObject::SyncAll(){
             // con << sn_SyncRequestedObject.Len() << "/" << sn_netObjects.Len() << "\n";
 
             int currentSync = sn_SyncRequestedObject.Len()-1;
-            while(sn_Connections[user].socket>0 &&
+            while(sn_Connections[user].socket &&
                     sn_Connections[user].bandwidthControl_.CanSend() &&
                     sn_Connections[user].ackPending<sn_maxNoAck &&
                     currentSync >= 0){
@@ -1778,30 +1792,33 @@ void nNetObject::ClearKnows(int user, bool clear){
     if (0<=user && user <=MAXCLIENTS){
         is_ready_to_get_objects[user]=false;
         for(int i=sn_netObjects.Len()-1;i>=0;i--){
-            nNetObject *no=sn_netObjects(i);
+            nNetObject* no=sn_netObjects(i);
             if (no){
+                nObserverPtr<nNetObject> noObserved{no};
                 no->knowsAbout[user].Reset();
 
                 no->DoBroadcastExistence();  // immediately transfer the thing
+                if(!noObserved)
+                    continue;
 
                 if (clear){
                     if (no->owner==user && user!=sn_myNetID){
 #ifdef DEBUG
                         sn_BreakOnObjectID(i);
 #endif
-                        if (no->ActionOnQuit())
+                        bool destroy = no->ActionOnQuit();
+                        if(!noObserved)
+                          continue;
+                        
+                        // take ownership of the object in any case
+                        no->createdLocally=true;
+                        no->owner=::sn_myNetID;
+                        sn_netObjectsOwner(i)=::sn_myNetID;
+
+                        if (destroy || no->AcceptClientSync())
                         {
-                            no->createdLocally=true;
                             tControlledPTR< nNetObject > bounce( no ); // destroy it, if noone wants it
                         }
-                        else
-                        {
-                            no->owner=::sn_myNetID; // or make it mine.
-                            sn_netObjectsOwner(i)=::sn_myNetID;
-                            if (no->AcceptClientSync()){
-                                tControlledPTR< nNetObject > bounce( no ); // destroy it, if noone wants it
-                            }
-                        }
                     }
                 }
             }
@@ -1961,6 +1978,13 @@ void sn_Sync(REAL timeout,bool sync_sn_netObjects, bool otherEnd){
                 if (sync_sn_netObjects)
                     nNetObject::SyncAll();
                 sn_SendPlanned();
+
+                // check for user abort, but just make it time out faster
+                if ( tConsole::Idle(false) )
+                {
+                    endTime -= 10;
+                    timeout *= .5;
+                }
             }
 
             // decrease timeout for next try
@@ -1998,6 +2022,12 @@ void sn_Sync(REAL timeout,bool sync_sn_netObjects, bool otherEnd){
                 }
             }
 
+            // check for user abort, but just make it time out faster
+            if ( tConsole::Idle(false) )
+            {
+                endTime -= 10;
+            }
+
             if (tSysTimeFloat()>endTime)
             {
                 goon=false;
