@@ -143,7 +143,7 @@ public:
 
     bool operator != ( const nVersion& other ){ return !operator==(other); }
     bool operator == ( const nVersion& other );
-    nVersion& operator = ( const nVersion& other );
+    nVersion& operator = ( const nVersion& other ) = default;
 private:
     int min_, max_;
 };
@@ -234,6 +234,7 @@ public:
     REAL GetAverage() const;               //!< returns the average value
     REAL GetDataVariance() const;          //!< returns the variance of the data ( average of (value-average)^2 )
     REAL GetAverageVariance() const;       //!< returns the expected variance of the returned average
+    REAL GetWeight() const;                //!< returns the current weight
     void Timestep( REAL decay );           //!< lets all values decay, so they can be replaced by new ones
     void Add( REAL value, REAL weight=1 ); //!< adds a value to the average
     void Reset();                          //!< resets average to zero
@@ -448,6 +449,10 @@ public:
         messageIDBig_ = 0;
     }
 
+    void BendMessageID( int id ){ // bends the message ID. Use with extreme caution.
+        messageIDBig_ = id;
+    }
+
     nMessage(const nDescriptor &);  // create a new message
     nMessage(unsigned short*& buffer, short sn_myNetID, int lenLeft );
     // read a message from the network stream
@@ -695,6 +700,7 @@ class nMachineDecorator: public tListItem< nMachineDecorator >
 {
 public:
     inline void Destroy();         //!< called when machine gets destroyed
+    virtual void OnBan();          //!< called when machine gets banned
 protected:
     virtual void OnDestroy();      //!< called when machine gets destroyed
 
@@ -704,6 +710,8 @@ private:
     nMachineDecorator();           //!< constructor
 };
 
+class nMachineIteratorPimpl;
+
 //! class trying to collect information about a certain client, persistent between connections
 class nMachine
 {
@@ -716,7 +724,21 @@ public:
     bool operator == ( nMachine const & other ) const; //!< equality operator
     bool operator != ( nMachine const & other ) const; //!< inequality operator
 
-    static nMachine & GetMachine( unsigned short userID ); //!< fetches the machine information of a user
+    class iterator
+    {
+    public:
+        nMachine & operator *() const;
+        iterator();
+        ~iterator();
+        void operator ++();
+        void operator ++(int);
+        bool Valid();
+    private:
+        nMachineIteratorPimpl * pimpl_;
+    };
+
+    static nMachine & GetMachine( unsigned short userID ); //!< fetches the machine information of a user, creating it on demand
+    static nMachine * PeekMachine( unsigned short userID ); //!< fetches the machine information of a user, returning NULL if none is found
     static void Expire();                       //!< expires machine information that is no longer needed
     static void KickSpectators();               //!< remove clients without players from the server
 
@@ -728,6 +750,9 @@ public:
     void     Ban( REAL time, tString const & reason ); //!< ban users from this machine for the given time
     REAL     IsBanned() const; //!< returns the number of seconds users from this machine are currently banned for
 
+    bool      IsValidated() const { return validated_; }
+    void      Validate()          { validated_ = true; }
+
     // player accounting
     void      AddPlayer();     //!< call when a player joins from this machine
     void      RemovePlayer();  //!< call when a player rom this machine leaves
@@ -744,6 +769,8 @@ private:
     int            players_;      //!< number of players coming from this machine currently
     REAL           lastPlayerAction_; //!< time of the last player action
 
+    bool           validated_;    //!< true if the machine has been validated as a real client without spoofed IP
+
     tString        IP_;           //!< IP address of the machine
     nMachineDecorator * decorators_; //!< list of decorators
 
@@ -794,7 +821,7 @@ public:
 };
 
 // on disconnection, this returns a server we should be redirected to (or NULL if we should not be redirected)
-std::auto_ptr< nServerInfoBase > sn_GetRedirectTo();
+std::unique_ptr< nServerInfoBase > sn_GetRedirectTo();
 
 // take a peek at the same info
 nServerInfoBase * sn_PeekRedirectTo();
