@@ -40,9 +40,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "nServerInfo.h"
 #include "nNetObject.h"
-#ifdef KRAWALL_SERVER
-#include "nAuthentication.h"
-#endif
 
 #include <fstream>
 
@@ -56,10 +53,6 @@ static bool                  sn_SendAll     [MAXCLIENTS+2];
 static bool                  sn_Requested   [MAXCLIENTS+2];
 static REAL                  sn_Timeout     [MAXCLIENTS+2];
 
-#ifdef KRAWALL_SERVER
-static bool                  sn_Auth        [MAXCLIENTS+2];
-#endif
-
 static nServerInfo*          sn_Requesting=NULL;
 static unsigned int          sn_NextTransactionNr = 0;
 
@@ -109,10 +102,6 @@ static void login_callback(){
     sn_SendAll     [nCallbackLoginLogout::User()] = true;
     sn_Requested   [nCallbackLoginLogout::User()] = false;
     sn_Timeout     [nCallbackLoginLogout::User()] = tSysTimeFloat() + 70;
-#ifdef KRAWALL_SERVER
-    sn_Auth        [nCallbackLoginLogout::User()] = false;
-#endif
-
 }
 
 // helper function: server info to string
@@ -123,23 +112,6 @@ tString ToString( const nServerInfoBase & info )
     return ret;
 }
 
-// authentification stuff
-#ifdef KRAWALL_SERVER_LEAGUE
-void ResultCallback(const tString& username,
-                    const tString& origUsername,
-                    int user, bool success)
-{
-    if (success)
-    {
-        sn_Auth[user] = true;
-        sn_Transmitting[user] = nServerInfo::GetFirstServer();
-    }
-    else
-        nAuthentication::RequestLogin(username, user, tOutput("$login_request_failed"), true);
-}
-#endif // KRAWALL_SERVER_LEAGUE
-
-
 
 static nCallbackLoginLogout nlc(&login_callback);
 
@@ -151,6 +123,32 @@ static nServerInfo *CreateServerInfo()
         return tNEW(nServerInfo());
 }
 
+static void sn_AddMasterServer( std::istream &s )
+{
+    tString connectionName;
+    unsigned port = 4533;
+    
+    s >> connectionName;
+    if ( !s.eof() )
+        s >> port;
+    
+    // back up regular server list
+    nServerInfo *oldFirstServer = sn_FirstServer;
+    sn_FirstServer = NULL;
+
+    nServerInfo *newMaster = CreateServerInfo();
+    newMaster->SetConnectionName( connectionName );
+    newMaster->SetPort( port );
+    newMaster->Remove();
+    newMaster->Insert( sn_masterList );
+    
+    // restore regular server list
+    sn_FirstServer = oldFirstServer;
+    nServerInfo::TellMasterAboutMe( newMaster );    
+}
+
+static tConfItemFunc sn_addMasterServerConfItemFunc( "ADD_MASTER_SERVER", &sn_AddMasterServer );
+
 nServerInfo::nServerInfo()
         :tListItem<nServerInfo>(sn_FirstServer),
         pollID(-1),
@@ -576,7 +574,7 @@ static void CheckDuplicate( nServerInfo * server )
     nServerInfo *run = nServerInfo::GetFirstServer();
     while(!IsDouble && run)
     {
-        if (run != server && *run == *server )
+        if (run != server && *run == *server)
             IsDouble = true;
         
         run = run->Next();
@@ -667,10 +665,10 @@ static nDescriptor BigServerDescriptor(51,nServerInfo::GetBigServerInfo,"big_ser
 static nDescriptor BigServerMasterDescriptor(54,nServerInfo::GetBigServerInfoMaster,"big_server_master", true);
 
 // request small server information from master server/broadcast
-static nDescriptor RequestSmallServerInfoDescriptor(52,nServerInfo::GiveSmallServerInfo,"small_request", true);
+nDescriptor RequestSmallServerInfoDescriptor(52,nServerInfo::GiveSmallServerInfo,"small_request", true);
 
 // request big server information from master server/broadcast
-static nDescriptor RequestBigServerInfoDescriptor(53,nServerInfo::GiveBigServerInfo,"big_request", true);
+nDescriptor RequestBigServerInfoDescriptor(53,nServerInfo::GiveBigServerInfo,"big_request", true);
 static nDescriptor RequestBigServerInfoMasterDescriptor(55,nServerInfo::GiveBigServerInfoMaster,"big_request_master", true);
 
 // used to transfer the rest of the server info (name, number of players, etc)
@@ -857,6 +855,7 @@ public:
     double lastTime_[MAX];   // log of the last times the server was pinged by this client
     int lastTimeIndex_;      // the current index in the array
     bool warned_;            // flag to avoid warning spam in the log file
+    REAL timeFactor_;        // locked time factor
 
     tString GetIP()
     {
@@ -913,24 +912,55 @@ public:
     // determines whether this client should be considered flooding
     bool FloodProtection( REAL timeFactor )
     {
+        if( warned_ && timeFactor < timeFactor_ )
+        {
+            // restore time factor
+            timeFactor = timeFactor_;
+        }
+        else
+        {
+            // store passed time factor for next time
+            timeFactor_ = timeFactor;
+        }
+
+
+
         int i;
         double now = tSysTimeFloat();
 
         bool protect = false;
         REAL diff = 0;
         int count = 0;
+        REAL tolerance = 0;
+        REAL minRelDiff = 10;
 
         // go through the different levels
-        for ( i = sn_minPingCount-1; i >= 0; --i )
+        int lowest = 0;
+
+        // only check 10 and 20 ping limit for the default timefactor
+        if( timeFactor < 1 && sn_minPingTimes[sn_minPingCount-1] > 0 )
+        {
+            // lowest = 2;
+        }
+
+        for ( i = sn_minPingCount-1; i >= lowest; --i )
         {
             // this many pings should be tracked
             count = sn_minPingCounts[i];
             diff = now - lastTime_[(lastTimeIndex_ + MAX - count) % MAX];
-            REAL tolerance = sn_minPingTimes[i]*timeFactor;
-            if ( tolerance > 0 && diff < tolerance )
+            tolerance = sn_minPingTimes[i]*timeFactor;
+            if ( tolerance > 0 )
             {
-                protect = true;
-                break;
+                if( tolerance*minRelDiff > diff )
+                {
+                    minRelDiff = diff/tolerance;
+                }
+
+                if( diff < tolerance )
+                {
+                    protect = true;
+                    break;
+                }
             }
         }
 
@@ -946,7 +976,7 @@ public:
         }
 
         // reset warning flag
-        if ( warned_ && now - lastTime_[(lastTimeIndex_ + MAX-2 ) % MAX] > sn_minPingTimes[ sn_minPingCount-1 ] )
+        if ( warned_ && minRelDiff > 4 )
         {
             con << "Flood protection ban of " << GetIP() << " removed.\n";
             warned_ = false;
@@ -991,16 +1021,23 @@ static nCallbackReceivedComplete sn_resetFirstInPacket( sn_ResetFirstInPacket );
 static REAL sn_minPingTimeGlobalFactor = 0.1;
 static tSettingItem< REAL > sn_minPingTimeGlobal( "PING_FLOOD_GLOBAL", sn_minPingTimeGlobalFactor );
 
+// checks for global ping flood events
+static bool GlobalPingFloodProtection()
+{
+    static nMachine server;
+
+    return sn_minPingTimeGlobalFactor > 0 && FloodProtection( server, sn_minPingTimeGlobalFactor );
+}
+
 // determines wheter the message comes from a flood attack; if so, reject it (return true)
 bool FloodProtection( nMessage const & m )
 {
-    // get the machine infos
-    nMachine & server = nMachine::GetMachine( 0 );
-    nMachine & peer   = nMachine::GetMachine( m.SenderID() );
-
     // only accept one ping per packet
     if ( !sn_firstInPacket )
     {
+        // get the machine infos
+        nMachine & peer   = nMachine::GetMachine( m.SenderID() );
+
         GetQueryMessageStats( peer ).Block();
 
         return true;
@@ -1012,11 +1049,15 @@ bool FloodProtection( nMessage const & m )
     if ( sn_minPingTimes[sn_minPingCount - 1] <= 0 )
         return false;
 
-    // and delegate
-    return FloodProtection( peer ) || ( sn_minPingTimeGlobalFactor > 0 && FloodProtection( server, sn_minPingTimeGlobalFactor ) );
+    // and delegate. Only do global check, the per-peer check has already been
+    // done earlier as the packet was received.
+    return GlobalPingFloodProtection();
 }
 
 void nServerInfo::GetSmallServerInfo(nMessage &m){
+    if ( !sn_IsMaster && sn_GetNetState() == nSERVER )
+        return;
+
     nServerInfoBase baseInfo;
     baseInfo.NetRead( m );
 
@@ -1139,24 +1180,7 @@ void nServerInfo::GiveSmallServerInfo(nMessage &m)
         con << "Giving server info to user " << m.SenderID() << "\n";
 
         sn_Requested[m.SenderID()] = true;
-#ifdef KRAWALL_SERVER_LEAGUE
-        // one moment! check if we need authentification
-        tString adr;
-        unsigned int port = sn_GetPort(m.SenderID());
-        sn_GetAdr(m.SenderID(), adr);
-        if (nKrawall::RequireMasterLogin(adr, port))
-        {
-            nAuthentication::SetLoginResultCallback(&ResultCallback);
-            nAuthentication::RequestLogin(tString(""), m.SenderID(), tOutput("$login_request_master"));
-        }
-        else
-        {
-            sn_Transmitting[m.SenderID()] = GetFirstServer();
-            sn_Auth[m.SenderID()]         = true;
-        }
-#else
         sn_Transmitting[m.SenderID()] = GetFirstServer();
-#endif
 
         if (m.End())
             sn_SendAll[m.SenderID()] = true;
@@ -1236,7 +1260,7 @@ nServerInfo* nServerInfo::GetBigServerInfoCommon(nMessage &m)
         {
             tOutput message;
             message.SetTemplateParameter(1, ToString( baseInfo ) );
-            message.SetTemplateParameter(2, sn_Connections[m.MessageID()].socket->GetAddress().ToString() );
+            message.SetTemplateParameter(2, sn_Connections[m.SenderID()].socket->GetAddress().ToString() );
             message << "$network_browser_unidentified";
             con << message;
         }
@@ -1272,6 +1296,9 @@ void nServerInfo::GiveBigServerInfoCommon(nMessage &m, const nServerInfo & info,
 
 void nServerInfo::GetBigServerInfo(nMessage &m)
 {
+    if ( !sn_IsMaster && sn_GetNetState() == nSERVER )
+        return;
+
     nServerInfo * server = GetBigServerInfoCommon( m );
 
     if (!server)
@@ -1299,6 +1326,19 @@ void nServerInfo::GiveBigServerInfo(nMessage &m)
     if (sn_IsMaster)
         return;
 
+    // log first polls
+    static int logPolls=10;
+    if( logPolls > 0 )
+    {
+        logPolls--;
+        tString sender;
+        sn_GetAdr( m.SenderID(), sender );
+        con << tOutput(
+            logPolls > 0 ? "$network_master_pollanswer" : "$network_master_pollanswer_last", 
+            sender
+            );
+    }
+
     // collect info
     nServerInfo me;
     me.GetFrom( sn_Connections[m.SenderID()].socket );
@@ -1321,7 +1361,7 @@ void nServerInfo::SetFromMaster()
 
 void nServerInfo::GetBigServerInfoMaster(nMessage &m)
 {
-    if ( sn_GetNetState() == nSERVER && FloodProtection( m ) )
+    if ( sn_GetNetState() == nSERVER )
         return;
 
     nServerInfo *server = GetBigServerInfoCommon( m );
@@ -1334,10 +1374,10 @@ void nServerInfo::GetBigServerInfoMaster(nMessage &m)
 
 void nServerInfo::GiveBigServerInfoMaster(nMessage &m)
 {
-    if ( FloodProtection( m ) )
+    if ( !sn_IsMaster )
         return;
 
-    if ( !sn_IsMaster )
+    if ( FloodProtection( m ) )
         return;
 
     // read info of desired server from message
@@ -1706,15 +1746,15 @@ void nServerInfo::TellMasterAboutMe(nServerInfoBase *masterInfo)
     // don't reinitialize the network system
     nSocketResetInhibitor inhibitor;
 
-    static unsigned int lastPort = 0;
+    // static unsigned int lastPort = 0;
 
     // enter server state so we know our true port number
     sn_SetNetState(nSERVER);
-    unsigned int port = sn_GetServerPort();
+    // unsigned int port = sn_GetServerPort();
     //if (port == lastPort)
     //    return; // the master already knows about us
 
-    lastPort = port;
+    // lastPort = port;
 
     //    sn_SetNetState(nSTANDALONE);
 
@@ -2216,22 +2256,23 @@ void nServerInfo::RunMaster()
         {
             // kick the user soon when the transfer is completed
             if ((sn_Requested[i] && !sn_Transmitting[i]
-#ifdef KRAWALL_SERVER
-                    && sn_Auth[i]
-#endif
                     && sn_MessagesPending(i) == 0))
             {
                 if (sn_Timeout[i] > tSysTimeFloat() + .2f)
                     sn_Timeout[i] = tSysTimeFloat() + .2f;
             }
 
-            // defend against DOS attacks: Kill idle clients
+            // defend against DoS attacks: Kill idle clients
             if(sn_Timeout[i] < tSysTimeFloat())
+            {
+                sn_DisconnectUser(i, "$network_kill_servercomplete");
+                continue;
+            }
+            else if (!sn_Requested[i] && sn_Timeout[i] < tSysTimeFloat() + 60.0f)
+            {
                 sn_DisconnectUser(i, "$network_kill_timeout");
-
-            if (!sn_Requested[i] && sn_Timeout[i] < tSysTimeFloat() + 60.0f)
-                sn_DisconnectUser(i, "$network_kill_timeout");
-
+                continue;
+            }
         }
 
         if (sn_Transmitting[i] && sn_MessagesPending(i) < 3)
@@ -2400,6 +2441,67 @@ nServerInfo::Compat	nServerInfo::Compatibility() const
     return Compat_Ok;
 }
 
+nServerInfo::SettingsDigest::SettingsDigest()
+  : flags_(0)
+  ,  minPlayTimeTotal_(0)
+  ,  minPlayTimeOnline_(0)
+  ,  minPlayTimeTeam_(0)
+  ,  cycleDelay_(0)
+  ,  acceleration_(0)
+  ,  rubberWallHump_(0)
+  ,  rubberHitWallRatio_(0)
+{
+}
+
+void nServerInfo::SettingsDigest::SetFlag( Flags flag, bool set )
+{
+    if( set )
+    {
+        flags_ |= flag;
+    }
+    else
+    {
+        flags_ &= ~flag;
+    }
+}
+
+bool nServerInfo::SettingsDigest::GetFlag( Flags flag ) const
+{
+    return 0 != (flags_ & flag);
+}
+
+//! callback to give other components a chance to help fill in the server info
+nServerInfo::SettingsDigest * nCallbackFillServerInfo::settings_ = NULL;
+static tCallback * sn_fillServerInfoAnchor = NULL;
+nCallbackFillServerInfo::nCallbackFillServerInfo( VOIDFUNC * f )
+: tCallback( sn_fillServerInfoAnchor, f )
+{}
+
+//! fills all server infos
+void nCallbackFillServerInfo::Fill( nServerInfo::SettingsDigest * settings )
+{
+    settings_ = settings;
+    Exec( sn_fillServerInfoAnchor );
+    settings_ = NULL;
+}
+
+nServerInfo::SettingsDigest const * nCallbackCanPlayOnServer::settings_ = NULL;
+static tCallbackString * sn_canPlayOnServerAnchor = NULL;
+//! callback to give other components a chance to help fill in the server info
+
+nCallbackCanPlayOnServer::nCallbackCanPlayOnServer( STRINGRETFUNC * f )
+: tCallbackString( sn_canPlayOnServerAnchor, f )
+{
+}
+
+//! return all reasons why you can't play here
+tString nCallbackCanPlayOnServer::CantPlayReasons( nServerInfo::SettingsDigest const * settings )
+{
+    settings_ = settings;
+    tString ret = Exec( sn_canPlayOnServerAnchor );
+    settings_ = NULL;
+    return ret;
+}
 
 static nServerInfoAdmin* sn_serverInfoAdmin = NULL;
 
@@ -2458,7 +2560,7 @@ nServerInfoBase::~nServerInfoBase()
 
 bool nServerInfoBase::operator ==( const nServerInfoBase & other ) const
 {
-    return GetAddress() == other.GetAddress() && port_ == other.port_;
+    return GetAddress().IsSet() && GetAddress() == other.GetAddress() && port_ == other.port_;
 }
 
 // *******************************************************************************************
@@ -2605,7 +2707,8 @@ void nServerInfoBase::NetReadThis( nMessage & m )
     m >> port_;                            // get the port
     sn_ReadFiltered( m, connectionName_ ); // get the connection name
 
-    if ( ( sn_IsMaster || sn_AcceptingFromBroadcast || sn_AcceptingFromMaster ) && connectionName_.Len()>1 ) // no valid name (must come directly from the server who does not know his own address)
+    // ban us.to dns service, it's down too often
+    if ( ( ( sn_IsMaster && !st_StringEndsWith( connectionName_, ".us.to" ) ) || sn_AcceptingFromBroadcast || sn_AcceptingFromMaster ) && connectionName_.Len()>1 ) // valid name (must come directly from the server who does not know his own address)
     {
         // resolve DNS
         connectionName_ = S_LocalizeName( connectionName_ );
@@ -2763,6 +2866,8 @@ void nServerInfo::DoGetFrom( nSocket const * socket )
         url_        = str;
         userGlobalIDs_ = "";
     }
+
+    nCallbackFillServerInfo::Fill( &settings_ );
 }
 
 // *******************************************************************************************
@@ -2790,6 +2895,16 @@ void nServerInfo::NetWriteThis( nMessage & m ) const
     m << url_;
 
     m << userGlobalIDs_;
+
+    m.Write(settings_.flags_);
+    m << settings_.minPlayTimeTotal_;
+    m << settings_.minPlayTimeOnline_;
+    m << settings_.minPlayTimeTeam_;
+    m << settings_.cycleDelay_;
+    m << settings_.acceleration_;
+    m << settings_.rubberWallHump_;
+    m << settings_.rubberHitWallRatio_;
+    m << settings_.wallsLength_;
 }
 
 // *******************************************************************************************
@@ -2852,6 +2967,31 @@ void nServerInfo::NetReadThis( nMessage & m )
         userGlobalIDs_ = "";
     }
 
+    if( !m.End() )
+    {
+        m.Read(settings_.flags_);
+        settings_.SetFlag(SettingsDigest::Flags_SettingsDigestSent, true);
+        m >> settings_.minPlayTimeTotal_;
+        m >> settings_.minPlayTimeOnline_;
+        m >> settings_.minPlayTimeTeam_;
+        m >> settings_.cycleDelay_;
+        m >> settings_.acceleration_;
+        m >> settings_.rubberWallHump_;
+        m >> settings_.rubberHitWallRatio_;
+        if( !m.End() )
+        {
+            m >> settings_.wallsLength_;
+        }
+        else
+        {
+            settings_.wallsLength_ = -1;
+        }
+    }
+    else
+    {
+        settings_.flags_ = 0;
+    }
+
     userNamesOneLine_.Clear();
     for ( int i = 0, j = 0; i < userNames_.Len()-1 ; ++i )
     {
@@ -2885,7 +3025,6 @@ void nServerInfo::NetReadThis( nMessage & m )
             if ( !advancedInfoSetEver )
             {
                 con << "Acknowledged server: " <<  ToString( *this ) << ", name: \"" << tColoredString::RemoveColors(name) << "\"\n";
-                con << "\n";
                 Save();
             }
             else if ( name != oldName )
@@ -2973,8 +3112,7 @@ nServerInfoBase const & nServerInfoBase::GetAddress( nAddress & address ) const
 
 nServerInfoBase const & nServerInfoBase::ClearAddress() const
 {
-    std::auto_ptr< nAddress > clearedAddress;
-    address_ = clearedAddress;
+    address_.reset();
     return *this;
 }
 
@@ -3011,13 +3149,11 @@ nAddress & nServerInfoBase::AccessAddress( void ) const
     // create address if it is not already there
     if ( !this->address_.get() )
     {
-        std::auto_ptr< nAddress > address( tNEW( nAddress ) );
+        this->address_.reset( tNEW( nAddress ) );
 
         // fill it with hostname and port
-        address->SetHostname( this->GetConnectionName() );
-        address->SetPort( this->GetPort() );
-
-        this->address_ = address;
+        address_->SetHostname( this->GetConnectionName() );
+        address_->SetPort( this->GetPort() );
 
 #ifdef DEBUG
         tString unresolved = ToString( *this );
