@@ -833,7 +833,7 @@ int ANET_GetSocketAddr (int sock, struct sockaddr *addr)
 }
 
 //=============================================================================
-
+/*
 int ANET_AddrCompare (struct sockaddr *addr1, struct sockaddr *addr2)
 {
     if (addr1->sa_family != addr2->sa_family)
@@ -847,7 +847,8 @@ int ANET_AddrCompare (struct sockaddr *addr1, struct sockaddr *addr2)
 
     return 0;
 }
-
+*/
+    
 }   // namespace
 
 //=============================================================================
@@ -1431,7 +1432,7 @@ bool nAddress::IsSet () const
 //!
 //!		@param	a1  first address to compare
 //!		@param	a2  second address to compare
-//!		@return
+//!		@return -1 if the addresses don't match, +1 if the ports don't match, 0 if everything matches.
 //!
 // *******************************************************************************************
 
@@ -1516,8 +1517,13 @@ int nSocket::Create( void )
     // initialize networking at OS level
     sn_InitOSNetworking();
 
+    int socktype = socktype_;
+#ifndef WIN32
+    socktype |= SOCK_CLOEXEC;
+#endif
+
     // open new socket
-    socket_ = socket( family_, socktype_, protocol_ );
+    socket_ = socket( family_, socktype, protocol_ );
     if ( socket_ < 0 )
         return -1;
 
@@ -1530,8 +1536,8 @@ int nSocket::Create( void )
 #endif    
 
     // unblock it
-    bool _true = true;
-    return ioctl (socket_, FIONBIO, reinterpret_cast<char *>(&_true)) == -1;
+    unsigned long _true = true;
+    return ioctl (socket_, FIONBIO, &_true) == -1;
 }
 
 // archives the binding procedure
@@ -2074,9 +2080,9 @@ int nSocket::Write( const int8 * buf, int len, const sockaddr * addr, int addrle
 
     // check if return value was archived
     static char const * section = "SEND";
+    static tReproducibleRandomizer randomizer;
     if ( !tRecorder::Playback( section, ret ) )
     {
-        static tReproducibleRandomizer randomizer;
 #ifdef DEBUG
         // pretend send was successful in packet loss simulation
         if ( sn_simulateSendPacketLoss > randomizer.Get() )
@@ -2628,35 +2634,36 @@ bool nBasicNetworkSystem::Select( REAL dt )
         if ( controlSocket_.GetSocket() < 0 )
         {
             tDelay( int( dt * 1000000 ) );
-            return false;
         }
-
-        fd_set rfds; // set of sockets to wathc
-        struct timeval tv; // time value to pass to select()
-
-        FD_ZERO( &rfds );
-
-        // watch the control socket
-        FD_SET( controlSocket_.GetSocket(), &rfds );
-        // con << "Watching " << controlSocket_.GetSocket();
-
-        int max = controlSocket_.GetSocket();
-
-        // watch listening sockets
-        for( nSocketListener::SocketArray::const_iterator iter = listener_.GetSockets().begin(); iter != listener_.GetSockets().end(); ++iter )
+        else
         {
-            FD_SET( (*iter).GetSocket(), &rfds );
-            if ( (*iter).GetSocket() > max )
-                max = (*iter).GetSocket();
-            // con << ", " << (*iter).GetSocket();
+            fd_set rfds; // set of sockets to watch
+            struct timeval tv; // time value to pass to select()
+
+            FD_ZERO( &rfds );
+
+            // watch the control socket
+            FD_SET( controlSocket_.GetSocket(), &rfds );
+            // con << "Watching " << controlSocket_.GetSocket();
+
+            int max = controlSocket_.GetSocket();
+
+            // watch listening sockets
+            for( nSocketListener::SocketArray::const_iterator iter = listener_.GetSockets().begin(); iter != listener_.GetSockets().end(); ++iter )
+            {
+                FD_SET( (*iter).GetSocket(), &rfds );
+                if ( (*iter).GetSocket() > max )
+                    max = (*iter).GetSocket();
+                // con << ", " << (*iter).GetSocket();
+            }
+
+            // set time
+            tv.tv_sec  = static_cast< long int >( dt );
+            tv.tv_usec = static_cast< long int >( (dt-tv.tv_sec)*1000000 );
+
+            // delegate to system select
+            retval = select(max+1, &rfds, NULL, NULL, &tv);
         }
-
-        // set time
-        tv.tv_sec  = static_cast< long int >( dt );
-        tv.tv_usec = static_cast< long int >( (dt-tv.tv_sec)*1000000 );
-
-        // delegate to system select
-        retval = select(max+1, &rfds, NULL, NULL, &tv);
     }
     tRecorder::Record( section, retval );
 
