@@ -210,6 +210,11 @@ tConfItemBase::tConfItemMap & tConfItemBase::ConfItemMap()
     return *st_confMap;
 }
 
+tConfItemBase::tConfItemMap const & tConfItemBase::GetConfItemMap()
+{
+    return ConfItemMap();
+}
+
 static bool st_preventCasacl = false;
 
 tCasaclPreventer::tCasaclPreventer( bool prevent )
@@ -223,6 +228,12 @@ tCasaclPreventer::~tCasaclPreventer()
     st_preventCasacl = previous_;
 }
 
+//! returns whether we're currently in an RINCLUDE file
+bool tCasaclPreventer::InRInclude()
+{
+    return st_preventCasacl;
+}
+
 // changes the access level of a configuration item
 class tConfItemLevel: public tConfItemBase
 {
@@ -262,7 +273,22 @@ public:
         {
             // and change the level
             tConfItemBase * ci = (*iter).second;
-            if ( ci->requiredLevel != level )
+
+            if( ci->requiredLevel < tCurrentAccessLevel::GetAccessLevel() )
+            {
+                con << tOutput( "$access_level_nochange_now", 
+                                name, 
+                                tCurrentAccessLevel::GetName( ci->requiredLevel ),
+                                tCurrentAccessLevel::GetName( tCurrentAccessLevel::GetAccessLevel() ) );
+            }
+            else if( level < tCurrentAccessLevel::GetAccessLevel() )
+            {
+                con << tOutput( "$access_level_nochange_later", 
+                                name, 
+                                tCurrentAccessLevel::GetName( level ),
+                                tCurrentAccessLevel::GetName( tCurrentAccessLevel::GetAccessLevel() ) );
+            }
+            else if ( ci->requiredLevel != level )
             {
                 ci->requiredLevel = level;
                 if(printChange)
@@ -289,6 +315,11 @@ public:
     virtual bool Save(){
         return false;
     }
+
+    // CAN this be saved at all?
+    virtual bool CanSave(){
+        return false;
+    }
 };
 
 static tConfItemLevel st_confLevel;
@@ -326,7 +357,7 @@ public:
         else if ( tCurrentAccessLevel::GetAccessLevel() > required )
         {
             con << tOutput( "$access_level_error",
-                            "SUDO",
+                            "CASACL",
                             tCurrentAccessLevel::GetName( required ),
                             tCurrentAccessLevel::GetName( tCurrentAccessLevel::GetAccessLevel() )
                 );
@@ -356,6 +387,11 @@ public:
     virtual bool Save(){
         return false;
     }
+
+    // CAN this be saved at all?
+    virtual bool CanSave(){
+        return false;
+    }
 };
 
 static tCasacl st_sudo;
@@ -450,7 +486,10 @@ int tConfItemBase::EatWhitespace(std::istream &s){
             !s.eof())
         c=s.get();
 
-    s.putback(c);
+    if( s.good() )
+    {
+        s.putback(c);
+    }
 
     return c;
 }
@@ -669,7 +708,7 @@ static bool s_VetoPlayback( tString const & line )
           "ZTRICK", "MOUSE_GRAB", "PNG_SCREENSHOT", // "WHITE_SPARKS", "SPARKS",
           "KEEP_WINDOW_ACTIVE", "TEXTURE_MODE", "TEXTURES_HI", "LAG_O_METER", "INFINITY_PLANE",
           "SKY_WOBBLE", "LOWER_SKY", "UPPER_SKY", "DITHER", "HIGH_RIM", "FLOOR_DETAIL",
-          "FLOOR_MIRROR", "SHOW_FPS", "TEXT_OUT", "SMOOTH_SHADING", "ALPHA_BLEND",
+          "FLOOR_MIRROR", "SHOW_FPS", "SHOW_RECORDING_TIME", "TEXT_OUT", "SMOOTH_SHADING", "ALPHA_BLEND",
           "PERSP_CORRECT", "POLY_ANTIALIAS", "LINE_ANTIALIAS", "FAST_FORWARD_MAXSTEP",
           "DEBUG_GNUPLOT", "FLOOR_", "MOVIEPACK_", "RIM_WALL_",
           0 };
@@ -794,11 +833,11 @@ void tConfItemBase::LoadAll(std::ifstream &s, bool record )
 
 //! @param s        file stream to be used for reading later
 //! @param filename name of the file to open
-//! @param var      whether to look in var directory
+//! @param path     whether to look in var directory
 //! @return success flag
 bool tConfItemBase::OpenFile( std::ifstream & s, tString const & filename, SearchPath path )
 {
-    bool ret = ( ( path & Config ) && tDirectories::Config().Open(s, filename ) ) || ( ( path & Var ) && tDirectories::Var().Open(s, filename ) );
+    bool ret = ( ( path & Config ) && tDirectories::Config().Open(s, filename ) ) || ( ( path & Var ) && st_StringEndsWith(filename, ".cfg") && tDirectories::Var().Open(s, filename ) );
     
     static char const * section = "INCLUDE_VOTE";
     tRecorder::Playback( section, ret );
@@ -1067,7 +1106,7 @@ static void Include(std::istream& s, bool error )
     if ( !tRecorder::IsPlayingBack() )
     {
         // really load include file
-        if ( !Load( tDirectories::Var(), file ) )
+        if ( !st_StringEndsWith(file, ".cfg") || !Load( tDirectories::Var(), file ) )
         {
             if (!Load( tDirectories::Config(), file ) && error )
             {
