@@ -20,7 +20,7 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-  
+
 ***************************************************************************
 
 */
@@ -31,6 +31,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "tList.h"
 #include "tString.h"
 #include "tLinkedList.h"
+#include "tException.h"
 #include "tLocale.h"
 #include "tConsole.h"
 #include "tLocale.h"
@@ -151,9 +152,82 @@ private:
  * The old stuff follows
  ************************************************************************/
 
+//! access levels for admin interfaces; lower numeric values are better
+enum tAccessLevel
+{
+    tAccessLevel_Owner = 0,        // the server owner
+    tAccessLevel_Admin = 1,        // one of his admins
+    tAccessLevel_Moderator = 2,    // one of the moderators
+    tAccessLevel_3 = 3,            // reserved
+    tAccessLevel_4 = 4,            // reserved
+    tAccessLevel_Armatrator = 5,   // reserved
+    tAccessLevel_6 = 6,            // reserved
+    tAccessLevel_TeamLeader = 7,   // a team leader
+    tAccessLevel_TeamMember = 8,   // a team member
+    tAccessLevel_9 = 9,            // reserved
+    tAccessLevel_10 = 10,          // reserved
+    tAccessLevel_11 = 11,          // reserved
+    tAccessLevel_Local      = 12,  // user with a local account
+    tAccessLevel_13 = 13,          // reserved
+    tAccessLevel_14 = 14,          // reserved
+    tAccessLevel_Remote = 15,      // user with remote account
+    tAccessLevel_DefaultAuthenticated = 15,     // default access level for authenticated users
+    tAccessLevel_FallenFromGrace = 16,          // authenticated, but not liked
+    tAccessLevel_Shunned = 17,          // authenticated, but disliked
+    tAccessLevel_18 = 18,          // reserved
+    tAccessLevel_Authenticated = 19,// any authenticated player
+    tAccessLevel_Program = 20,     // a regular player
+    tAccessLevel_21 = 21,          // reserved
+    tAccessLevel_22 = 22,          // reserved
+    tAccessLevel_23 = 23,          // reserved
+    tAccessLevel_24 = 24,          // reserved
+    tAccessLevel_25 = 25,          // reserved
+    tAccessLevel_Invalid = 255,    // completely invalid level
+    tAccessLevel_Default = 20
+};
+
+//! class to temporarily allow/forbid the use of casacl
+class tCasaclPreventer
+{
+public:
+    tCasaclPreventer( bool prevent = true );
+    ~tCasaclPreventer();
+
+    static bool InRInclude(); //!< returns whether we're currently in an RINCLUDE file
+private:
+    bool previous_; //!< previous value of prevention flag
+};
+
+//! class managing the current access level
+class tCurrentAccessLevel
+{
+    friend class tCasacl;
+public:
+    //! for the lifetime of this object, change the user's admit level to the passed one.
+    tCurrentAccessLevel( tAccessLevel newLevel, bool allowElevation = false );
+
+    //! does not change the access level on construction, but resets it on destruction
+    tCurrentAccessLevel();
+    ~tCurrentAccessLevel();
+
+    //! returns the current access level
+    static tAccessLevel GetAccessLevel();
+
+    //! returns the name of an access level
+    static tString GetName( tAccessLevel level );
+private:
+    tCurrentAccessLevel( tCurrentAccessLevel const & );
+    tCurrentAccessLevel & operator = ( tCurrentAccessLevel const & );
+
+    tAccessLevel lastLevel_; //!< used to restore the last admin level when the object goes out of scope
+    static tAccessLevel currentLevel_; //!< the current access level
+};
+
 class tConfItemBase
 {
     friend class tCheckedPTRBase;
+    friend class tConfItemLevel;
+    friend class tAccessLevelSetter;
 
     int id;
 protected:
@@ -161,12 +235,19 @@ protected:
     const tOutput help;
     bool changed;
 
+    tAccessLevel requiredLevel; //!< access level required to change this setting
+    tAccessLevel setLevel;      //!< access level of the user making the last change to this setting
+
+public:
+    // the map of all configuration items
     typedef std::map< tString, tConfItemBase * > tConfItemMap;
+    static tConfItemMap const & GetConfItemMap();
+protected:
     static tConfItemMap & ConfItemMap();
+public:
 
     // static tConfItemBase* s_ConfItemAnchor;
     //static tConfItemBase* Anchor(){return dynamic_cast<tConfItemBase *>(s_ConfItemAnchor);}
-public:
     static bool printChange; //!< if set, setting changes are printed to the console and, if printErrors is set as well, suggestions of typo fixes are given.
     static bool printErrors; //!< if set, unknown settings are pointed out.
 
@@ -174,24 +255,59 @@ public:
     tConfItemBase(const char *title);
     virtual ~tConfItemBase();
 
-    tString const & GetTitle() const { return title; }
+    tString const & GetTitle() const {
+        return title;
+    }
+
+    tAccessLevel GetRequiredLevel() const { return requiredLevel; }
+    tAccessLevel GetSetLevel() const { return setLevel; }
 
     static int EatWhitespace(std::istream &s); // eat whitespace from stream; return: first non-whitespace char
 
     static void SaveAll(std::ostream &s);
-    static void LoadAll(std::istream &s);  //! loads configuration from file
+    static void LoadAll(std::istream &s, bool record = false );  //! loads configuration from stream
+    static void LoadAll(std::ifstream &s, bool record = false );  //! loads configuration from file
     static void LoadLine(std::istream &s); //! loads one configuration line
     static bool LoadPlayback( bool print = false ); //! loads configuration from playback
     static void DocAll(std::ostream &s);
 
+    // helper functions for files (use these, they manage recording and playback properly)
+    enum SearchPath
+    {
+        Config = 1,
+        Var    = 2,
+        All    = 3
+    };
+
+    static bool OpenFile( std::ifstream & s, tString const & filename, SearchPath path ); //! opens a file stream for configuration reading
+    static void ReadFile( std::ifstream & s ); //! loads configuration from a file
+
     virtual void ReadVal(std::istream &s)=0;
     virtual void WriteVal(std::ostream &s)=0;
 
     virtual void WasChanged(){} // what to do if a read changed the thing
 
-    virtual bool Writable(){return true;}
+    virtual bool Writable(){
+        return true;
+    }
 
-    virtual bool Save(){return true;}
+    // should this be saved into user.cfg?
+    virtual bool Save(){
+        return true;
+    }
+
+    // CAN this be saved at all?
+    virtual bool CanSave(){
+        return true;
+    }
+};
+
+//! just to do some work in static initializers, to modify default access levels:
+class tAccessLevelSetter
+{
+public:
+    //! modifies the access level of <item> to <level>
+    tAccessLevelSetter( tConfItemBase & item, tAccessLevel level );
 };
 
 // Arg! Msvc++ could not handle bool IO. Seems to be fine now.
@@ -226,24 +342,51 @@ template<> struct tTypeToConfig< TYPE > \
     typedef STREAM TOSTREAM; \
   typedef int * DUMMYREQUIRED; \
 } \
-
+ 
 //! macro for configuration enums: convert them to int.
 #define tCONFIG_ENUM( TYPE ) tCONFIG_AS( TYPE, int )
 
+tCONFIG_ENUM( tAccessLevel );
+
+//! exception to be thrown when the current script should be aborted
+class tAbortLoading: public tException
+{
+public:
+    tAbortLoading( char const * command );
+private:
+    tString command_; //!< the command responsible for the abort
+
+    virtual tString DoGetName() const;
+    virtual tString DoGetDescription() const;
+};
+
 template<class T> class tConfItem:virtual public tConfItemBase{
+public:
+    typedef bool (*ShouldChangeFuncT)(T const &newValue);
 protected:
     T    *target;
+    ShouldChangeFuncT shouldChangeFunc_;
 
-    tConfItem(T &t):tConfItemBase(""),target(&t){};
+    tConfItem(T &t):tConfItemBase(""),target(&t), shouldChangeFunc_(NULL) {}
 public:
     tConfItem(const char *title,const tOutput& help,T& t)
-            :tConfItemBase(title,help),target(&t){}
+            :tConfItemBase(title,help),target(&t), shouldChangeFunc_(NULL) {}
 
     tConfItem(const char *title,T& t)
-            :tConfItemBase(title),target(&t){}
+            :tConfItemBase(title),target(&t), shouldChangeFunc_(NULL) {}
+        
+    tConfItem(const char*title, T& t, ShouldChangeFuncT changeFunc)
+            :tConfItemBase(title),target(&t),shouldChangeFunc_(changeFunc) {}
 
     virtual ~tConfItem(){}
 
+    tConfItem<T> & SetShouldChangeFunc( ShouldChangeFuncT changeFunc )
+    
+    {
+        this->shouldChangeFunc_ = changeFunc;
+        return *this;
+    }
+
     typedef typename tTypeToConfig< T >::DUMMYREQUIRED DUMMYREQUIRED;
 
     // read without conversion
@@ -292,22 +435,29 @@ public:
                     {
                         tOutput o;
                         o.SetTemplateParameter(1, title);
-                        o << "nconfig_errror_protected";
+                        o << "$nconfig_error_protected";
                         con << "";
                     }
                     else{
-                        if (printChange)
+                        if (!shouldChangeFunc_ || shouldChangeFunc_(dummy))
                         {
-                            tOutput o;
-                            o.SetTemplateParameter(1, title);
-                            o.SetTemplateParameter(2, *target);
-                            o.SetTemplateParameter(3, dummy);
-                            o << "$config_value_changed";
-                            con << o;
-                        }
+                            if (printChange)
+                            {
+                                tOutput o;
+                                o.SetTemplateParameter(1, title);
+                                o.SetTemplateParameter(2, *target);
+                                o.SetTemplateParameter(3, dummy);
+                                o << "$config_value_changed";
+                                con << o;
+                            }
 
-                        *target=dummy;
-                        changed=true;
+                            *target = dummy;
+                            changed = true;
+                        }
+                        else
+                        {
+                            con << tOutput("$config_value_not_changed", title, *target, dummy);
+                        }
                     }
                 }
         }
@@ -322,7 +472,7 @@ public:
 
         // read the rest of the line
         c=' ';
-        while(c!='\n' && s.good() && !s.eof()) c=s.get();
+        while (c!='\n' && s.good() && !s.eof()) c=s.get();
     }
 
     virtual void WriteVal(std::ostream &s){
@@ -337,22 +487,27 @@ public:
 
     tSettingItem(const char *title,T& t)
             :tConfItemBase(title),tConfItem<T>(title, t){}
+    
+    tSettingItem(const char *title, T& t, typename tConfItem<T>::ShouldChangeFuncT changeFunc)
+            :tConfItemBase(title), tConfItem<T>(title, t, changeFunc) {}
 
     virtual ~tSettingItem(){}
 
-    virtual bool Save(){return false;}
+    virtual bool Save(){
+        return false;
+    }
 };
 
 
 class tConfItemLine:public tConfItem<tString>, virtual public tConfItemBase{
 public:
     tConfItemLine(const char *title,const char *help,tString &s)
-            :tConfItemBase(title,help),tConfItem<tString>(title,help,s){};
+            :tConfItemBase(title,help),tConfItem<tString>(title,help,s){}
 
     virtual ~tConfItemLine(){}
 
     tConfItemLine(const char *title, tString &s)
-            :tConfItemBase(title),tConfItem<tString>(title,s){};
+            :tConfItemBase(title),tConfItem<tString>(title,s){}
 
     virtual void ReadVal(std::istream &s);
     virtual void WriteVal(std::ostream &s);
