@@ -455,7 +455,7 @@ char *eh_getdir(const char *da, size_t *len) {
 tString expand_home(tString const & pathname) {
     const char *pn = (const char *)pathname;
     char *s;
-    size_t len;
+    size_t len = 0;
     tString r;
 
     if ((pn[0] == '~' || !strncmp(pn, "${", 2)) && (s = eh_getdir(pn, &len))) {
@@ -931,6 +931,7 @@ bool tDirectories::FileMatchesWildcard(const char *str, const char *pattern,
             if (*pattern)
                 c = *pattern++;
 
+            // fallthrough, we're just unescaping
         default:
             if (ignoreCase)
             {
@@ -1277,13 +1278,15 @@ public:
         char const * bestGuess = "./" PROGNAME;
 #endif// win32
 
-#ifndef ENABLE_BINRELOC
-        // if the passed default path is a real path, let it override the best guess
-        if ( strstr( defaultPath, "/" ) || strstr( defaultPath, "\\" ) )
-            bestGuess = defaultPath;
-        //            bestGuess = "./armagetronad-dedicated";
+#ifdef ENABLE_BINRELOC
+	if ( !bestGuess || 0 == strlen(bestGuess) )
 #endif
-
+        {
+            // if the passed default path is a real path, let it override the best guess
+            if ( strstr( defaultPath, "/" ) || strstr( defaultPath, "\\" ) )
+                bestGuess = defaultPath;
+            //            bestGuess = "./armagetronad-dedicated";
+        }
         path_ = bestGuess;
 
 #ifdef DEBUG_PATH
@@ -1323,6 +1326,8 @@ static tString GeneratePrefix()
     tString const & bindirCompiled = st_bindirCompiled;
     // and the current binary path
     tString bindirNow(GenerateParentOfExecutable(1));
+    if(bindirNow.Len() <= 1)
+        return prefixCompiled;
 
     // the length of the bindir suffix, the part that is added below prefix
     int bindirSuffixLength=bindirCompiled.Len() - prefixCompiled.Len();
@@ -1560,6 +1565,17 @@ tString tPath::GetPaths(void) const {
     return ret;
 }
 
+tString tPath::GetPaths(char const * delimiter, char const * finalizer) const {
+    tString ret;
+    tArray<tString> paths;
+    Paths(paths);
+    for (int i = 0; i < paths.Len(); ++i) {
+        ret << paths[i];
+        ret << ( (i == paths.Len() - 1) ? finalizer : delimiter );
+    }
+    return ret;
+}
+
 void st_PrintPathInfo(tOutput &buf) {
     tString const hcol("0xff8888");
     buf << hcol << "$path_info_user_cfg"   << "0xRESETT\n   " << tDirectories::Var().GetReadPath("user.cfg") << "\n"
