@@ -82,6 +82,27 @@ bool tRecorderBase::IsRunning( void )
     return IsRecording() || IsPlayingBack();
 }
 
+// *****************************************************************************************
+// *
+// *   Stop
+// *
+// *****************************************************************************************
+//!
+//!
+// *****************************************************************************************
+
+void tRecorderBase::StopRecording( void )
+{
+    return tRecording::Stop();
+}
+static void st_StopRecording(std::istream &)
+{
+    tRecorderBase::StopRecording();
+}
+
+static tConfItemFunc snm("STOP_RECORDING",&st_StopRecording);
+
+
 // *******************************************************************************************
 // *
 // *    Record
@@ -417,6 +438,8 @@ bool tPlaybackBlockBase::Initialize( char const * section, tPlayback * playback
     if (!playback_)
         return false;
 
+    // std::cout << playback_->GetNextSection() << "," << section << "\n";
+
     // read section
     if( playback_->GetNextSection() != section )
     {
