@@ -2301,7 +2301,7 @@ bool gAIPlayer::EmergencySurvive( ThinkData & data, int enemyevade, int prefered
         if (character->properties[AI_ENEMY] > 7)
         {
             // would he be able to trap us if we drive straight on?
-            bool trap[2] = {false, false};
+            // bool trap[2] = {false, false};
 
             if (!isTrapped)
                 for (i = 1; i>=0; i--)
@@ -2324,7 +2324,7 @@ bool gAIPlayer::EmergencySurvive( ThinkData & data, int enemyevade, int prefered
                     if (loop)
                         if (winding * (i+i-1) < 0)
                         {
-                            trap[i] = true;
+                            // trap[i] = true;
                             REAL x = enemypos.x * (i+i-1);
                             REAL y = enemypos.y;
 
@@ -2781,8 +2781,8 @@ REAL gAIPlayer::Think(){
     // get the sensors to the left and right with the most free space
     int currentDirectionNumber = Object()->Grid()->DirectionWinding( dir );
     REAL mindistLeft = 1E+30, mindistRight = 1E+30;
-    std::auto_ptr< gAISensor > left  ( sg_GetSensor( currentDirectionNumber, *Object(), -1, side, range, corridor, mindistLeft ) );
-    std::auto_ptr< gAISensor > right ( sg_GetSensor( currentDirectionNumber, *Object(), 1, side, range, corridor, mindistRight ) );
+    std::unique_ptr< gAISensor > left  ( sg_GetSensor( currentDirectionNumber, *Object(), -1, side, range, corridor, mindistLeft ) );
+    std::unique_ptr< gAISensor > right ( sg_GetSensor( currentDirectionNumber, *Object(), 1, side, range, corridor, mindistRight ) );
 
     // count intermediate walls to the left and right as if they were in front
     {
