@@ -20,7 +20,7 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-  
+
 ***************************************************************************
 
 */
@@ -40,8 +40,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 //#include "eTess.h"
 #include "rTexture.h"
 #include "tConfiguration.h"
+#include "tRandom.h"
 #include "tRecorder.h"
 #include "tCommandLine.h"
+#include "tToDo.h"
 #include "eAdvWall.h"
 #include "eGameObject.h"
 #include "uMenu.h"
@@ -51,7 +53,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "gCycle.h"
 //#include <unistd>
 #include <stdio.h>
-#include <stdlib.h> 
+#include <stdlib.h>
 #include <fstream>
 
 #include "nServerInfo.h"
@@ -62,12 +64,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "rSDL.h"
 #endif
 
-
 // data structure for command line parsing
 class gMainCommandLineAnalyzer: public tCommandLineAnalyzer
 {
 public:
-    bool     daemon_;
     bool     fullscreen_;
     bool     windowed_;
     bool     use_directx_;
@@ -75,7 +75,6 @@ public:
 
     gMainCommandLineAnalyzer()
     {
-        daemon_ = false;
         windowed_ = false;
         fullscreen_ = false;
         use_directx_ = false;
@@ -86,11 +85,7 @@ public:
 private:
     virtual bool DoAnalyze( tCommandLineParser & parser )
     {
-        if ( parser.GetSwitch( "--daemon","-d") )
-        {
-            daemon_ = true;
-        }
-        else if ( parser.GetSwitch( "-fullscreen", "-f" ) )
+        if ( parser.GetSwitch( "-fullscreen", "-f" ) )
         {
             fullscreen_=true;
         }
@@ -98,7 +93,6 @@ private:
         {
             windowed_=true;
         }
-
 #ifdef WIN32
         else if ( parser.GetSwitch( "+directx") )
         {
@@ -127,26 +121,147 @@ private:
         << "                               initialisation under MS Windows\n\n";
         s << "\n\nYes, I know this looks ugly. Sorry about that.\n";
 #endif
-#else
-#ifndef WIN32
-        s << "-d, --daemon                 : allow the dedicated server to run as a daemon\n"
-        << "                               (will not poll for input on stdin)\n";
 #endif
-#endif              
     }
 };
 
 static gMainCommandLineAnalyzer commandLineAnalyzer;
 
-static bool use_directx=true;
+extern bool sr_useDirectX; // rScreen.cpp
 #ifdef WIN32
 static tConfItem<bool> udx("USE_DIRECTX","makes use of the DirectX input "
                            "fuctions; causes some graphic cards to fail to work (VooDoo 3,...)",
-                           use_directx);
+                           sr_useDirectX);
 #endif
 
 extern void exit_game_objects(eGrid *grid);
 
+enum gConnection
+{
+    gLeave,
+    gDialup,
+    gISDN,
+    gDSL
+    // gT1
+};
+
+// initial setup menu
+void sg_StartupPlayerMenu()
+{
+    uMenu firstSetup("$first_setup", false);
+    firstSetup.SetBot(-.2);
+    
+    uMenuItemExit e2(&firstSetup, "$menuitem_accept", "$menuitem_accept_help");
+    
+    ePlayer * player = ePlayer::PlayerConfig(0);
+    tASSERT( player );
+
+    gConnection connection = gDSL;
+
+    uMenuItemSelection<gConnection> net(&firstSetup, "$first_setup_net", "$first_setup_net_help", connection );
+    if ( !st_FirstUse )
+    {
+        net.NewChoice( "$first_setup_leave", "$first_setup_leave_help", gLeave );
+        connection = gLeave;
+    }
+    net.NewChoice( "$first_setup_net_dialup", "$first_setup_net_dialup_help", gDialup );
+    net.NewChoice( "$first_setup_net_isdn", "$first_setup_net_isdn_help", gISDN );
+    net.NewChoice( "$first_setup_net_dsl", "$first_setup_net_dsl_help", gDSL );
+
+    tString keyboardTemplate("keys_cursor.cfg");
+    uMenuItemSelection<tString> k(&firstSetup, "$first_setup_keys", "$first_setup_keys_help", keyboardTemplate );
+    if ( !st_FirstUse )
+    {
+        k.NewChoice( "$first_setup_leave", "$first_setup_leave_help", tString("") );
+        keyboardTemplate="";
+    }
+    k.NewChoice( "$first_setup_keys_cursor", "$first_setup_keys_cursor_help", tString("keys_cursor.cfg") );
+    k.NewChoice( "$first_setup_keys_wasd", "$first_setup_keys_wasd_help", tString("keys_wasd.cfg") );
+    k.NewChoice( "$first_setup_keys_zqsd", "$first_setup_keys_zqsd_help", tString("keys_zqsd.cfg") );
+    k.NewChoice( "$first_setup_keys_cursor_single", "$first_setup_keys_cursor_single_help", tString("keys_cursor_single.cfg") );
+    // k.NewChoice( "$first_setup_keys_both", "$first_setup_keys_both_help", tString("keys_twohand.cfg") );
+    k.NewChoice( "$first_setup_keys_x", "$first_setup_keys_x_help", tString("keys_x.cfg") );
+
+    tColor leave(0,0,0,0);
+    tColor color(1,0,0);
+    uMenuItemSelection<tColor> c(&firstSetup,
+                                 "$first_setup_color",
+                                 "$first_setup_color_help",
+                                 color);   
+
+    if ( !st_FirstUse )
+    {
+        color = leave;
+        c.NewChoice( "$first_setup_leave", "$first_setup_leave_help", leave );
+    }
+
+    c.NewChoice( "$first_setup_color_red", "", tColor(1,0,0) );
+    c.NewChoice( "$first_setup_color_blue", "", tColor(0,0,1) );
+    c.NewChoice( "$first_setup_color_green", "", tColor(0,1,0) );
+    c.NewChoice( "$first_setup_color_yellow", "", tColor(1,1,0) );
+    c.NewChoice( "$first_setup_color_orange", "", tColor(1,.5,0) );
+    c.NewChoice( "$first_setup_color_purple", "", tColor(.5,0,1) );
+    c.NewChoice( "$first_setup_color_magenta", "", tColor(1,0,1) );
+    c.NewChoice( "$first_setup_color_cyan", "", tColor(0,1,1) );
+    c.NewChoice( "$first_setup_color_white", "", tColor(1,1,1) );
+    c.NewChoice( "$first_setup_color_dark", "", tColor(0,0,0) );
+    
+    if ( st_FirstUse )
+    {
+        for(int i=tRandomizer::GetInstance().Get(4); i>=0; --i)
+        {
+            c.LeftRight(1);
+        }
+    }
+
+    uMenuItemString n(&firstSetup,
+                      "$player_name_text",
+                      "$player_name_help",
+                      player->name, 16);
+    
+    uMenuItemExit e(&firstSetup, "$menuitem_accept", "$menuitem_accept_help");
+    
+    firstSetup.Enter();
+
+    // apply network rates
+    switch(connection)
+    {
+    case gDialup:
+        sn_maxRateIn  = 6;
+        sn_maxRateOut = 4;
+        break;
+    case gISDN:
+        sn_maxRateIn  = 8;
+        sn_maxRateOut = 8;
+        break;
+    case gDSL:
+        sn_maxRateIn  = 64;
+        sn_maxRateOut = 16;
+        break;
+    case gLeave:
+        break;
+    }
+
+    // store color
+    if( ! (color == leave) )
+    {
+        player->rgb[0] = int(color.r_*15);
+        player->rgb[1] = int(color.g_*15);
+        player->rgb[2] = int(color.b_*15);
+    }
+
+    // load keyboard layout
+    if( keyboardTemplate.Len() > 1 )
+    {
+        std::ifstream s;
+        if( tConfItemBase::OpenFile( s, keyboardTemplate, tConfItemBase::Config ) )
+        {
+            tCurrentAccessLevel level( tAccessLevel_Owner, true );
+            tConfItemBase::ReadFile( s );
+        }
+    }
+}
+
 #ifndef DEDICATED
 static void welcome(){
     bool textOutBack = sr_textOut;
@@ -176,7 +291,6 @@ static void welcome(){
 
     if (st_FirstUse)
     {
-        st_FirstUse=false;
         sr_LoadDefaultConfig();
         textOutBack = sr_textOut;
         sr_textOut = false;
@@ -207,7 +321,7 @@ static void welcome(){
             timeout = tSysTimeFloat() + 6;
 
             uInputProcessGuard inputProcessGuard;
-            while((!su_GetSDLInput(tEvent) || tEvent.type!=SDL_KEYDOWN) &&
+            while ((!su_GetSDLInput(tEvent) || tEvent.type!=SDL_KEYDOWN) &&
                     tSysTimeFloat() < timeout)
             {
                 if ( sr_glOut )
@@ -229,7 +343,7 @@ static void welcome(){
         // catch some keyboard input
         {
             uInputProcessGuard inputProcessGuard;
-            while (su_GetSDLInput(tEvent));
+            while (su_GetSDLInput(tEvent)) ;
         }
 
         sr_textOut = textOutBack;
@@ -246,63 +360,27 @@ static void welcome(){
     }
     rSysDep::SwapGL();
 
-    sg_LanguageMenu();
+    sr_textOut = textOutBack;
+    sg_StartupLanguageMenu();
 
-    // catch some keyboard input
-    {
-        uInputProcessGuard inputProcessGuard;
-        while (su_GetSDLInput(tEvent));
-    }
+    sr_textOut = textOutBack;
+    sg_StartupPlayerMenu();
 
-    timeout = tSysTimeFloat() + 10;
+    st_FirstUse=false;
 
-    sr_UnlockSDL();
-    uInputProcessGuard inputProcessGuard;
-    while((!su_GetSDLInput(tEvent) || tEvent.type!=SDL_KEYDOWN) &&
-            tSysTimeFloat() < timeout){
+    sr_textOut = textOutBack;
+    uMenu::Message( tOutput("$welcome_message_heading"), tOutput("$welcome_message"), 300 );
 
-        sr_ResetRenderState(true);
-        rViewport::s_viewportFullscreen.Select();
+    // start a first single player game
+    sg_currentSettings->speedFactor = -2;
+    sg_currentSettings->autoNum = 0;
+    sr_textOut = textOutBack;
+    sg_SinglePlayerGame();
+    sg_currentSettings->autoNum = 1;
+    sg_currentSettings->speedFactor = 0;
 
-        if ( sr_glOut )
-        {
-            rSysDep::ClearGL();
-
-            uMenu::GenericBackground();
-
-            REAL w=16*2/640.0;
-            REAL h=32*2/480.0;
-
-
-            //REAL middle=-.6;
-
-            Color(1,1,1);
-            DisplayText(0,.8,w,h,tOutput("$welcome_message_heading"));
-
-            w/=2;
-            h/=2;
-
-            rTextField c(-.8,.6, w, h);
-
-
-            c << tOutput("$welcome_message_intro");
-
-            c.SetIndent(12);
-
-            c << tOutput("$welcome_message_vendor")   << gl_vendor   << '\n';
-            c << tOutput("$welcome_message_renderer") << gl_renderer << '\n';
-            c << tOutput("$welcome_message_version")  << gl_version  << '\n';
-
-            c.SetIndent(0);
-
-            c << tOutput("$welcome_message_finish");
-
-            rSysDep::SwapGL();
-        }
-
-        tAdvanceFrame();
-    }
-    sr_LockSDL();
+    sr_textOut = textOutBack;
+    uMenu::Message( tOutput("$welcome_message_2_heading"), tOutput("$welcome_message_2"), 300 );
 
     sr_textOut = textOutBack;
 }
@@ -361,6 +439,12 @@ void cleanup(eGrid *grid){
 }
 
 #ifndef DEDICATED
+static bool sg_active = true;
+static void sg_DelayedActivation()
+{
+    Activate( sg_active );
+}
+
 int filter(const SDL_Event *tEvent){
     // recursion avoidance
     static bool recursion = false;
@@ -399,12 +483,12 @@ int filter(const SDL_Event *tEvent){
             tRecorder::Record("END");
 
             st_SaveConfig();
-            uMenu::quickexit=true;
+            uMenu::quickexit=uMenu::QuickExit_Total;
             return false;
         }
 
-        if(tEvent->type==SDL_MOUSEMOTION)
-            if(tEvent->motion.x==sr_screenWidth/2 && tEvent->motion.y==sr_screenHeight/2)
+        if (tEvent->type==SDL_MOUSEMOTION)
+            if (tEvent->motion.x==sr_screenWidth/2 && tEvent->motion.y==sr_screenHeight/2)
                 return 0;
         if (su_mouseGrab &&
                 tEvent->type!=SDL_MOUSEBUTTONDOWN &&
@@ -418,14 +502,24 @@ int filter(const SDL_Event *tEvent){
         if (tEvent->type==SDL_ACTIVEEVENT)
         {
             // Jonathans fullscreen bugfix.
-#ifdef MACOSX 
-            if(currentScreensetting.fullscreen ^ lastSuccess.fullscreen) return false;
-#endif 
+#ifdef MACOSX
+            if (currentScreensetting.fullscreen ^ lastSuccess.fullscreen) return false;
+#endif
             int flags = SDL_APPINPUTFOCUS;
-            if ( tEvent->active.gain && tEvent->active.state & flags )
-                Activate(true);
-            if ( !tEvent->active.gain && tEvent->active.state & flags )
-                Activate(false);
+            if ( tEvent->active.state & flags )
+            {
+                // con << tSysTimeFloat() << " " << "active: " << (tEvent->active.gain ? "on" : "off") << "\n";
+                sg_active = tEvent->active.gain;
+                st_ToDo(sg_DelayedActivation);
+            }
+
+            // reload GL stuff if application gets reactivated
+            if ( tEvent->active.gain && tEvent->active.state & SDL_APPACTIVE )
+            {
+                // just treat it like a screen mode change, gets the job done
+                st_ToDo(rCallbackBeforeScreenModeChange::Exec);
+                st_ToDo(rCallbackAfterScreenModeChange::Exec);
+            }
             return false;
         }
 
@@ -453,7 +547,39 @@ void sg_SetIcon()
 #endif
 }
 
-static const char * dedicatedSection = "DEDICATED";
+class gAutoStringArray
+{
+public:
+    ~gAutoStringArray()
+    {
+#ifdef HAVE_CLEARENV
+        // Optional. Systems that don't have this function better make copies of putenv() arguments.
+        clearenv();
+#endif
+
+        for ( std::vector< char * >::iterator i = strings.begin(); i != strings.end(); ++i )
+        {
+            free( *i );
+        }
+    }
+
+    char * Store( char const * s )
+    {
+        char * ret = strdup( s );
+        strings.push_back( ret );
+        return ret;
+    }
+private:
+    std::vector< char * > strings; // the stored raw C strings
+};
+
+// wrapper for putenv, taking care of the peculiarity that the argument
+// is kept in use for the rest of the program's lifetime
+void sg_PutEnv( char const * s )
+{
+    static gAutoStringArray store;
+    putenv( store.Store( s ) );
+}
 
 int main(int argc,char **argv){
     //std::cout << "enter\n";
@@ -475,14 +601,26 @@ int main(int argc,char **argv){
         if ( ! commandLine.Analyse(argc, argv) )
             return 0;
 
-        // read RAND_MAX from recording or write it to recording
-        if ( !tRecorder::PlaybackStrict( dedicatedSection, dedicatedServer ) )
+
         {
-#ifdef DEDICATED          
-            dedicatedServer = true;
+            // embed version in recording
+            const char * versionSection = "VERSION";
+            tString version( sn_programVersion );
+            tRecorder::Playback( versionSection, version );
+            tRecorder::Record( versionSection, version );
+        }
+
+        {
+            // read/write server/client mode from/to recording
+            const char * dedicatedSection = "DEDICATED";
+            if ( !tRecorder::PlaybackStrict( dedicatedSection, dedicatedServer ) )
+            {
+#ifdef DEDICATED
+                dedicatedServer = true;
 #endif
+            }
+            tRecorder::Record( dedicatedSection, dedicatedServer );
         }
-        tRecorder::Record( dedicatedSection, dedicatedServer );
 
         // tERR_MESSAGE( "Initializing player data." );
         ePlayer::Init();
@@ -500,15 +638,19 @@ int main(int argc,char **argv){
         if ( commandLineAnalyzer.windowed_ )
             currentScreensetting.fullscreen   = false;
         if ( commandLineAnalyzer.use_directx_ )
-            use_directx                       = true;
+            sr_useDirectX                       = true;
         if ( commandLineAnalyzer.dont_use_directx_ )
-            use_directx                       = false;
+            sr_useDirectX                       = false;
 
         gAICharacter::LoadAll(tString( "aiplayers.cfg" ) );
 
         sg_LanguageInit();
         atexit(tLocale::Clear);
 
+        static eLadderLogWriter sg_encodingWriter( "ENCODING", true );
+        sg_encodingWriter << st_internalEncoding;
+        sg_encodingWriter.write();
+
         if ( commandLine.Execute() )
         {
             gCycle::PrivateSettings();
@@ -531,11 +673,11 @@ int main(int argc,char **argv){
                 if (! tDirectories::Var().Open( s, "scorelog.txt", std::ios::app ) )
                 {
                     char const * error = "var directory not writable or does not exist. It should reside inside your user data directory and should have been created automatically on first start, but something must have gone wrong."
-                    #ifdef WIN32
+#ifdef WIN32
                                          " You can access your user data directory over one of the start menu entries we installed."
-                    #else
+#else
                                          " Your user data directory is subdirectory named .armagetronad in your home directory."
-                    #endif
+#endif
                                          ;
 
                     tERR_ERROR( error );
@@ -551,42 +693,49 @@ int main(int argc,char **argv){
                 }
             }
 
-		#ifndef DEDICATED
+#ifndef DEDICATED
             sr_glOut=1;
-		#endif
+#endif
 
             //std::cout << "checked mp\n";
 
             // while DGA mouse is buggy in XFree 4.0:
-		#ifdef linux
+#ifdef linux
             // Sam 5/23 - Don't ever use DGA, we don't need it for this game.
             if ( ! getenv("SDL_VIDEO_X11_DGAMOUSE") ) {
-                putenv("SDL_VIDEO_X11_DGAMOUSE=0");
+                sg_PutEnv( "SDL_VIDEO_X11_DGAMOUSE=0" );
             }
-		#endif
+#endif
 
-		#ifdef WIN32
+#ifdef WIN32
             // disable DirectX by default; it causes problems with some boards.
-            if (!use_directx && !getenv("SDL_VIDEODRIVER") ) {
-                putenv("SDL_VIDEODRIVER=windib");
+            if (!getenv( "SDL_VIDEODRIVER") ) {
+                if (sr_useDirectX) {
+                    // NO. Causes crashes with SDL1.2 on Windows on window deactivation/alt-tabbing away
+                    // sg_PutEnv( "SDL_VIDEODRIVER=directx" );
+                }
+                else
+                {
+                    sg_PutEnv( "SDL_VIDEODRIVER=windib" );
+                }
             }
-		#endif
+#endif
 
             // atexit(ANET_Shutdown);
 
-		#ifndef WIN32
-		#ifdef DEBUG
-            #define NOSOUND
-		#endif
-		#endif
+#ifndef WIN32
+#ifdef DEBUG
+#define NOSOUND
+#endif
+#endif
 
-		#ifndef DEDICATED
+#ifndef DEDICATED
             if (
-		#ifndef NOSOUND
-        #ifndef DEFAULT_SDL_AUDIODRIVER
+#ifndef NOSOUND
+#ifndef DEFAULT_SDL_AUDIODRIVER
                 SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO) < 0 &&
-        #endif
-		#endif
+#endif
+#endif
                 SDL_Init(SDL_INIT_VIDEO) < 0 )            {
                 tERR_ERROR("Couldn't initialize SDL: " << SDL_GetError());
             }
@@ -601,16 +750,17 @@ int main(int argc,char **argv){
             sg_SetIcon();
 
             tConsole::RegisterMessageCallback(&uMenu::Message);
+            tConsole::RegisterIdleCallback(&uMenu::IdleInput);
 
-		#ifndef NOSOUND
+#ifndef NOSOUND
             se_SoundInit();
             atexit(se_SoundExit);
-		#ifndef DEBUG
+#ifndef DEBUG
             // double sound initialisation for dodgy cards
             se_SoundExit();
             se_SoundInit();
-		#endif
-		#endif
+#endif
+#endif
 
             if (sr_InitDisplay()){
 
@@ -644,7 +794,6 @@ int main(int argc,char **argv){
                     nNetObject::ClearAll();
 
                     rITexture::UnloadAll();
-                    sr_RendererCleanup();
                 }
                 catch (tException const & e)
                 {
@@ -657,6 +806,7 @@ int main(int argc,char **argv){
                 }
 
                 sr_ExitDisplay();
+                sr_RendererCleanup();
 
                 //std::cout << "exit\n";
 
@@ -669,17 +819,14 @@ int main(int argc,char **argv){
             }
             se_SoundExit();
             SDL_Quit();
-		#else
-            if (!commandLineAnalyzer.daemon_)
-                sr_Unblock_stdin();
-
+#else
             sr_glOut=0;
 
             //  nServerInfo::TellMasterAboutMe();
 
             while (!uMenu::quickexit)
                 sg_HostGame();
-		#endif
+#endif
             nNetObject::ClearAll();
             nServerInfo::DeleteAll();
         }
@@ -689,13 +836,17 @@ int main(int argc,char **argv){
 
         //	tLocale::Clear();
     }
-    catch( tException const & e )
+    catch ( tCleanQuit const & e )
+    {
+        return 0;
+    }
+    catch ( tException const & e )
     {
         try
         {
             st_PresentError( e.GetName(), e.GetDescription() );
         }
-        catch(...)
+        catch (...)
         {
         }
 
@@ -705,20 +856,20 @@ int main(int argc,char **argv){
 #pragma warning ( disable : 4286 )
     // GRR. Visual C++ dones not handle generic exceptions with the above general statement.
     // A specialized version is needed. The best part: it warns about the code below being redundant.
-    catch( tGenericException const & e )
+    catch ( tGenericException const & e )
     {
         try
         {
             st_PresentError( e.GetName(), e.GetDescription() );
         }
-        catch(...)
+        catch (...)
         {
         }
 
         return 1;
     }
 #endif
-    catch(...)
+    catch (...)
     {
         return 1;
     }
@@ -728,7 +879,10 @@ int main(int argc,char **argv){
 
 #ifdef DEDICATED
 // settings missing in the dedicated server
-static void st_Dummy(std::istream &s){tString rest; rest.ReadLine(s);}
+static void st_Dummy(std::istream &s){
+    tString rest;
+    rest.ReadLine(s);
+}
 static tConfItemFunc st_Dummy10("MASTER_QUERY_INTERVAL", &st_Dummy);
 static tConfItemFunc st_Dummy11("MASTER_SAVE_INTERVAL", &st_Dummy);
 static tConfItemFunc st_Dummy12("MASTER_IDLE", &st_Dummy);
