@@ -40,7 +40,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "gGame.h"
 #include "tRecorder.h"
 
-#include <rRender.h>
+#include "rRender.h"
 #include <math.h>
 #include "gCycle.h"
 #include <time.h>
@@ -124,6 +124,9 @@ void GLmeter_subby(float value,float max, float locx, float locy, float size, co
     }
 }
 
+static REAL sg_hudCacheThreshold=.0001;
+static tSettingItem< REAL > sg_minDropIntervalConf( "HUD_CACHE_THRESHOLD", sg_hudCacheThreshold );
+
 class gGLMeter
 {
 public:
@@ -139,7 +142,7 @@ public:
         REAL change = rel - oldRel_;
 
         // see if the gauge change is enough to warrant an update
-        if ( change * change * ( time - oldTime_ ) > .000001 || time < oldTime_ )
+        if ( change * change * ( time - oldTime_ ) > sg_hudCacheThreshold*sg_hudCacheThreshold || time < oldTime_ )
         {
             list_.Clear();
         }
@@ -309,7 +312,7 @@ static void display_hud_subby( ePlayer* player ){
         if(subby_ShowHUD){
             // static int max=0,lastmax=0;
             // static int imax=-1,lastimax=0;
-            static REAL imax=-1;
+            // static REAL imax=-1;
             char fasteststring[50];
             // REAL distance;
             static float maxmeterspeed= 50;
@@ -317,11 +320,11 @@ static void display_hud_subby( ePlayer* player ){
             static REAL max=max_player_speed;
             static tString name;
             static tString ultiname;
-            static bool  wrotefastest = false;
-            static bool wrote10 =false;
+            // static bool  wrotefastest = false;
+            // static bool wrote10 =false;
             static bool belowzero=false;
             static REAL ultimax=0;
-            static REAL timelast;
+            // static REAL timelast;
             static REAL myscore, topscore;
             // tString realname = ePlayer::PlayerConfig(0)->Name();
 
@@ -346,10 +349,10 @@ static void display_hud_subby( ePlayer* player ){
                             //  con << "testcon";
                             //  sn_CenterMessage(message);
                         }else{
-                            timelast = se_GameTime();
+                            // timelast = se_GameTime();
                             max=0;
-                            wrotefastest = false;
-                            wrote10 =false;
+                            // wrotefastest = false;
+                            // wrote10 =false;
                         }
                     }else if(se_GameTime()>0&&belowzero){
                         belowzero=false;
@@ -372,7 +375,7 @@ static void display_hud_subby( ePlayer* player ){
                         if (h->Speed()>max){
                             max =  (float) h->Speed();  // changed to float for more accuracy in reporting top speed
                             name = p->GetName();
-                            imax = i;
+                            // imax = i;
 
                         }
                         if( h->Speed()>ultimax){
@@ -529,7 +532,7 @@ static void display_fps_subby()
     float size =.15;
     rTextField c2(.7,.85,.15*size, .3*size);
 
-    if ( tRecorder::IsRunning() )
+    if ( sr_RecordingTimeOut && tRecorder::IsRunning() )
     {
         std::stringstream time;
         time.precision( 5 );
@@ -552,7 +555,7 @@ static void display_fps_subby()
     // Show the time
     if(showTime) {
         static int lastTime=0;
-        static char theTime[13];
+        static char theTime[13*3];
         float size =.15;
         struct tm* thisTime;
         time_t rawtime;
@@ -561,9 +564,9 @@ static void display_fps_subby()
         thisTime = localtime ( &rawtime );
 
         if(thisTime->tm_min != lastTime) {
-            char h[3];
-            char m[3];
-            char ampm[3] = " ";
+            char h[13];
+            char m[13];
+            char ampm[13] = " ";
 
             lastTime = thisTime->tm_min;
 
