@@ -147,8 +147,10 @@ static rNoAutoDisplayAtNewlineCallback su_noNewline( uMenu::MenuActive );
 void uMenu::OnEnter(){
 #ifndef DEDICATED
     float nextrepeat = 0.0f;
-    static const float repeatdelay = 0.3f;
-    static const float repeatrate  = 0.05f;
+    static const float repeatdelay = 0.2f;
+    static const float repeatrateStart  = 0.2f;
+    static const float repeatrateMin  = 0.05f;
+    static float repeatrate  = repeatrateStart;
     SDL_Event tEventRepeat;
 #else
     return;
@@ -170,7 +172,7 @@ void uMenu::OnEnter(){
 
 #ifndef DEDICATED
     lastkey=tSysTimeFloat();
-    static const REAL timeout=3;
+    static const REAL timeout=.5;
 #endif
 
     while (!exitFlag && !quickexit && !exitToMain){
@@ -212,6 +214,7 @@ void uMenu::OnEnter(){
                     break;
                 case SDL_KEYUP:
                     repeat = false;
+                    repeatrate = repeatrateStart;
                     break;
                 }
 
@@ -231,6 +234,9 @@ void uMenu::OnEnter(){
             {
                 this->HandleEvent( tEventRepeat );
                 nextrepeat = tSysTimeFloat() + repeatrate;
+                repeatrate *= .71;
+                if ( repeatrate < repeatrateMin )
+                    repeatrate = repeatrateMin;
             }
         }
 
@@ -319,20 +325,26 @@ void uMenu::OnEnter(){
             if (YPos(menuentries-1)>menuTop && (int(tSysTimeFloat())+1)%2)
                 arrow(.9,menuTop,1,.05);
 
-            if (tSysTimeFloat()-lastkey>timeout){
-                disphelp=true;
+            REAL helpAlpha = tSysTimeFloat()-lastkey-timeout;
+            if( helpAlpha > 1 )
+            {
+                helpAlpha = 1;
+            }
+            
+            disphelp = helpAlpha > 0;
+            if ( items[selected]->DisplayHelp( disphelp, menuBot, helpAlpha ) )
+            {
                 if (sr_alphaBlend)
-                    glColor4f(1,.8,.8,tSysTimeFloat()-lastkey-timeout);
+                    glColor4f(1,.8,.8, helpAlpha );
                 else
-                    Color(tSysTimeFloat()-lastkey-timeout,
-                          .8*(tSysTimeFloat()-lastkey-timeout),
-                          .8*(tSysTimeFloat()-lastkey-timeout));
+                    Color(helpAlpha,
+                          .8*helpAlpha,
+                          .8*helpAlpha);
 
                 rTextField c(-.95f,menuBot-.04f);
                 c.SetWidth(static_cast<int>((1.9f-items[selected]->SpaceRight())/c.GetCWidth()));
                 c << items[selected]->Help();
             }
-            else disphelp=false;
         }
         else
 #endif
@@ -459,23 +471,67 @@ void uMenu::HandleEvent( SDL_Event event )
 #endif
 }
 
+#ifndef DEDICATED
+static bool s_idleBackground = false;
+#endif
 
 // paints a nice background
-void uMenu::GenericBackground(){
+void uMenu::GenericBackground(REAL top){
 #ifndef DEDICATED
     if (idle)
     {
+        s_idleBackground = true;
+
         try
         {
             // throw tGenericException("test"); // (test exception throw to see if error handling works right)
             (*idle)();
+
+            // render the console so it appears behind the menu
+            if( sr_con.autoDisplayAtSwap )
+            {
+                sr_con.Render();
+            }
+
+            // fade everything rendered so far to black
+            if( sr_alphaBlend )
+            {
+                sr_ResetRenderState(true);
+
+                double time = tSysTimeFloat();
+                static double lastTime = time - 100;
+                static REAL alpha = 0.0f;
+                double timePassed = time - lastTime;
+                if( time - lastTime > 1.0 )
+                {
+                    alpha = 0.0f;
+                }
+                else
+                {
+                    alpha += timePassed;
+                    static const REAL limit = .5;
+
+                    if( alpha > limit )
+                    {
+                        alpha = limit;
+                    }
+                }
+                lastTime = time;
+
+                RenderEnd();
+                glColor4f(0, 0, 0, alpha);
+                glRectf(-1,-1,1,top);
+            }
         }
         catch ( ... )
         {
+            s_idleBackground = false;
+
             // the idle background function is broken. Disable it and rethrow.
             idle = 0;
             throw;
         }
+        s_idleBackground = false;
     }
     else if (sr_glOut){
         uCallbackMenuBackground::MenuBackground();
@@ -1097,12 +1153,20 @@ void uCallbackMenuBackground::MenuBackground(){
 }
 
 // poll input, return true if ESC was pressed
-bool uMenu::IdleInput()
+bool uMenu::IdleInput( bool processInput )
 {
 #ifndef DEDICATED
+    if( !processInput )
+    {
+        sr_LockSDL();
+        SDL_PumpEvents();
+        sr_UnlockSDL();
+        return uMenu::quickexit != uMenu::QuickExit_Off;
+    }
+
     SDL_Event event;
     uInputProcessGuard inputProcessGuard;
-    while (su_GetSDLInput(event))
+    while (!s_idleBackground && su_GetSDLInput(event))
     {   
         switch (event.type)
         {
@@ -1121,6 +1185,8 @@ bool uMenu::IdleInput()
             break;
         }
     }   
+
+    return uMenu::quickexit != uMenu::QuickExit_Off;
 #endif
 
     return false;
